<?php


/**
 *
 * @ver 0.2.0
 */

class kstats_datetime {

	private $stopwatch;

	public $offset;

	public $start_of_week;

	public $timestamp;

	public $today;

	public $last_week;

	public $yesterday;

	public $this_month;

	/**
	 * Constructor, sets all class properties on object creation
	 * @ignore
	 */

	public function __construct() {

		$this->offset         = get_option( 'gmt_offset' );
		$this->start_of_week  = get_option( 'start_of_week' );

		$this->timestamp      = $this->get_timestamp();
		$this->today          = date( 'Y-m-d', $this->timestamp );
		$this->last_week      = $this->get_time_last_x_days( 7 );
		$this->yesterday      = $this->last_week[1];
		$this->this_month     = substr( $this->today, 0, 7 ); 

	}


	private function get_timestamp() {
		list( $month, $day, $year, $hour, $minute, $second ) = explode( ' ', gmdate( 'n j Y G i s' ) );
		return mktime( $hour + $this->offset, $minute, $second, $month, $day, $year );
	}

	/**
	 * Timer function, records the start time and returns it
	 * @ignore
	 */

	public function start_timer() {
		$mtime = explode( ' ', microtime() );
		$this->stopwatch = $mtime[1] + $mtime[0];
	}


	/**
	 * Timer function, based on the provided start time returns the total time elapsed as a string
	 * @ignore
	 */

	public function end_timer( $ech = TRUE, $msg = 'This page was created in %s seconds' ) {
		$mtime = explode( ' ', microtime() );
		$total = ( ( $mtime[1] + $mtime[0] ) - $this->stopwatch );

		return ( $ech ) ? printf( $msg, $total ) : sprintf( $msg, $total );
	}


	/**
	 * Builds an array of the last X days, including the current
	 * day for use by the Cleanup functions and other data sets that
	 * need to span periods of time.
	 *
	 * @ver 1.0.0
	 * @category Datetime
	 * @return array The array of the last X days
	 */

	public function get_time_last_x_days( $x = 21 ) {

		$last_x_days = array();

		$last_x_days[] = date( 'Y-m-d', $this->timestamp );

		for ( $i = 1; $i <= $x; $i++ )
			$last_x_days[] = date( 'Y-m-d', strtotime( '-' . $i . ' days', $this->timestamp ) );

		return $last_x_days;
	}


	/**
	 * Builds an array of the last X months, including the current
	 * month at index 0
	 *
	 * @ver 1.0.0
	 * @category Datetime
	 * @return array The array of the last X months
	 */

	public function get_time_last_x_months( $x = 6 ) {

		$last_x_months = array();

		$last_x_months[] = date( 'Y-m', $this->timestamp );

		for ( $i = 1; $i <= $x; $i++ )
			$last_x_months[] = date( 'Y-m', strtotime( '-' . $i . ' months', $this->timestamp ) );

		return $last_x_months;
	}

}