<?php



/**
 * Using the new settings API for Wordpress, let's register.
 * @ignore
 */

function kstats_options_init() {
	register_setting( 'kstats_admin_options', 'kstats_options', 'kstats_options_validate' );
}

function kstats_validate_range( $input, $high ) {
	if ( ! is_numeric( $input ) || $input < 1 || $input > $high )
		return $high;

	return $input;
}



function kstats_options_validate( $arr ) {
	global $kstats;

	$arr['config_access_minimum_to_view'] = ( array_key_exists( $arr['config_access_minimum_to_view'], $kstats->access_levels ) ? $arr['config_access_minimum_to_view'] : 'Subscriber' );
	$arr['config_access_minimum_to_edit'] = ( array_key_exists( $arr['config_access_minimum_to_edit'], $kstats->access_levels ) ? $arr['config_access_minimum_to_edit'] : 'Subscriber' );

	$arr['settings_database_months_to_keep']      = kstats_validate_range( $arr['settings_database_months_to_keep'], 24 );
	$arr['settings_database_days_to_keep']        = kstats_validate_range( $arr['settings_database_days_to_keep'], 365 );
	$arr['settings_database_rows_to_keep']        = 2000;


	$arr['config_dashboard_display_widget'] = ( $arr['config_dashboard_display_widget'] == 1 ) ? 1 : 0;

	$arr['config_dashboard_days_to_show']       = kstats_validate_range( $arr['config_dashboard_days_to_show'], 21 );
	$arr['config_overview_days_to_show']        = kstats_validate_range( $arr['config_overview_days_to_show'], 60 );
	$arr['config_overview_recent_hits_to_show'] = kstats_validate_range( $arr['config_overview_recent_hits_to_show'], 500 );

	if ( ! is_array( $arr['config_ignore_ignore_list'] ) )
		$arr['config_ignore_ignore_list'] = explode( "\r\n", $arr['config_ignore_ignore_list'] );

	foreach ( $arr['config_ignore_ignore_list'] as $key => $value ) {
		$value = kstats_validate_ip( $value );

		if ( $value === FALSE )
			unset( $arr['config_ignore_ignore_list'][$key] );
	}

	return $arr;
}



class kstats_reloaded {

	public $blog_url;

	public $datetime;
	public $version;

	public $settings;
	public $config;

	public $upgrade;

	private $upgrades = array( '0.3.1',
							   '0.4.1',
							   '0.4.2',
							   '0.6.0' );

	public $access_levels = array( 'administrator' => 'create_users',
								   'editor' => 'publish_pages',
								   'author' => 'publish_posts',
								   'contributor' => 'edit_posts',
								   'subscriber' => 'read'
							      );

	public function __construct() {

		$this->blog_url = get_bloginfo( 'url' );

		$this->datetime = new kstats_datetime;

		$this->version = get_option( 'kstats_version' );

		// I've changed version control for this WAY too much since 0.1.0
		// Sticking with the current model foreverrrrr now, but this needs to check against legacy models for now
		if ( $this->version === FALSE )
			$this->version = '0.3.1';

		$this->refresh_options();
		$this->upgrade = $this->_check_upgrades();

		if ( $this->upgrade !== FALSE ) {
			if ( substr( $_GET['page'], 0, 6 ) != 'kstats' )
				add_action( 'admin_notices', array( $this, '_upgrade_notice' ) );
		}

	}


	public function _upgrade_notice() {
		echo '<div class="error"><p><strong>' . __( 'kStats Upgrade Required!', 'kstats' ) . '</strong> ' . __( 'Please navigate to one of the kStats pages from the menu on your left and click on the "Upgrade Now!" button.', 'kstats' ) . '</p></div>';
	}

	private function _check_upgrades() {

		if ( version_compare( $this->version, KSTATS_VERSION, '<' ) ) {

			if ( array_key_exists( $this->version, $this->upgrades ) ) {
				return $this->version;
			}
			else {

				foreach ( $this->upgrades as $version ) {
					if ( version_compare( $version, $this->version, '>' ) )
						return $version;
				}

				// No special upgrade required, go ahead and update the version
				update_option( 'KSTATS_VERSION', KSTATS_VERSION );

			}

		}

		return FALSE;

	}


	public function refresh_options() {

		if ( $options = get_option( 'kstats_options' ) ) {

			// Hack to fix upgrade problem...
			if ( isset( $options['config_ignore_ignore_list_2'] ) ) {
				$options['config_ignore_ignore_list'] = $options['config_ignore_ignore_list_2'];
				unset( $options['config_ignore_ignore_list_2'] );
				update_option( 'kstats_options', $options );
			}

			foreach ( $options as $option => $value ) {
				$parts = explode( '_', $option );
		
				$description = implode( '_', array_slice( $parts, 2 ) );
		
				$this->{$parts[0]}[$parts[1]][$description] = $value;
			}

		}
		else {
			if ( ! defined( 'KSTATS_DISABLED' ) )
				define( 'KSTATS_DISABLED', 1 );
		}

	}




	public function set_notice( $type, $notice ) {
		$this->notices[$type] = $notice;
	}

}