<?php

/**
 * Responsible for completing our nightly aggregate of information. Runs our
 * helper methods first to gather their cumulative information on the untouched
 * raw stats, checks whether or not we're 
 *
 * @category Cleanup
 */

function kstats_cleanup() {
	global $wpdb, $kstats;

	kstats_cleanup_search_terms();
	kstats_cleanup_referrers();
	kstats_cleanup_agents();
	kstats_cleanup_spiders();

	$arr = kstats_get_totals();

	$query_parts[] = sprintf( "( 'totals', '', '%d', '%d', '%d', '%d' )", $arr['totals']['visitors'], $arr['totals']['pageviews'], $arr['totals']['spiders'], $arr['totals']['feeds'] );

	foreach ( $arr['monthly'] as $month => $data )
		$query_parts[] = sprintf( "( 'monthly', '%s', '%d', '%d', '%d', '%d' )", $month, $data['visitors'], $data['pageviews'], $data['spiders'], $data['feeds'] );

	foreach ( $arr['daily'] as $day => $data )
		$query_parts[] = sprintf( "( 'daily', '%s', '%d', '%d', '%d', '%d' )", $day, $data['visitors'], $data['pageviews'], $data['spiders'], $data['feeds'] );


	// Just in case, we don't need any collisions...
	$query[] = sprintf( "TRUNCATE TABLE `%s`", KSTATS_TABLE_TOTALS );
	$wpdb->query( end( $query ) );

	// Drop that data!
	$query[] = sprintf( "REPLACE INTO `%s` VALUES %s", KSTATS_TABLE_TOTALS, implode( ', ', $query_parts ) );
	$wpdb->query( end( $query ) );


	if ( isset( $kstats->settings['database']['days_to_keep'] ) && $kstats->settings['database']['days_to_keep'] != '0' ) {
		$last_day = $kstats->datetime->get_time_last_x_days( $kstats->settings['database']['days_to_keep'] );

		$query[] = sprintf( "DELETE FROM `%s` WHERE type = 'daily' AND name < '%s'", KSTATS_TABLE_TOTALS, $last_day[$kstats->settings['database']['days_to_keep']] );
		$wpdb->query( end( $query ) );
	}

	if ( isset( $kstats->settings['database']['months_to_keep'] ) && $kstats->settings['database']['months_to_keep'] != '0' ) {
		$last_month = $kstats->datetime->get_time_last_x_months( $kstats->settings['database']['months_to_keep'] );

		$query[] = sprintf( "DELETE FROM `%s` WHERE type = 'monthly' AND name < '%s'", KSTATS_TABLE_TOTALS, $last_month[$kstats->settings['database']['months_to_keep']] );
		$wpdb->query( end( $query ) );
	}


	if ( isset( $kstats->settings['database']['rows_to_keep'] ) && $kstats->settings['database']['rows_to_keep'] != '0' ) {

		$tempkey = kstats_create_temp_key();

		$query[] = sprintf( 'CREATE TEMPORARY TABLE `%1$s%2$s` SELECT * FROM `%1$s` ORDER BY timestamp DESC LIMIT %3$d', KSTATS_TABLE_RAW, $tempkey, ( ctype_digit( $kstats->settings['database']['rows_to_keep'] ) ? $kstats->settings['database']['rows_to_keep'] : '2000' ) );
		$wpdb->query( end( $query ) );

		$query[] = sprintf( "TRUNCATE TABLE `%s`", KSTATS_TABLE_RAW );
		$wpdb->query( end( $query ) );

		$query[] = sprintf( 'INSERT INTO `%1$s` SELECT * FROM `%1$s%2$s` ORDER BY timestamp ASC', KSTATS_TABLE_RAW, $tempkey );
		$wpdb->query( end( $query ) );

	}

	// Make sure we don't count this data, EVER AGAIN! Buahahaha...
	$query[] = sprintf( "UPDATE `%s` SET preserved = 1", KSTATS_TABLE_RAW );
	$wpdb->query( end( $query ) );
	
}


/**
 * Responsible for updating the kStats Top Searches aggregate information.
 * Using a temporary table, the function retrieves its data in the same method as
 * used by the kStats Views, and replaces the current date in our totals table to
 * reflect the updated information.
 *
 * @category Cleanup
 */

function kstats_cleanup_search_terms() {
	global $wpdb;

	$tempkey = kstats_create_temp_key();

	$query[] = sprintf( 'CREATE TEMPORARY TABLE `%1$s%2$s`
					     SELECT %3$d AS chart, SUM( v.count ) AS count, v.search_terms, v.search_engine, v.url, v.referrer
					     FROM ( SELECT count( search_terms ) AS count, search_terms, search_engine, url, referrer
							    FROM `%4$s`
							    WHERE search_terms <> \'\' AND preserved IS NULL
							    GROUP BY search_terms
							    UNION ALL
							    SELECT count, search_terms, search_engine, url, referrer
							    FROM `%1$s`
								WHERE chart = %3$d )
					     AS v
					     GROUP BY v.search_terms
					     ORDER BY count DESC', KSTATS_TABLE_CHARTS, $tempkey, KSTATS_CHARTS_SEARCHES, KSTATS_TABLE_RAW );

	$query[] = sprintf( "DELETE FROM `%s` WHERE chart = %d", KSTATS_TABLE_CHARTS, KSTATS_CHARTS_SEARCHES );

	$query[] = sprintf( 'INSERT INTO `%1$s` ( chart, count, search_terms, search_engine, url, referrer ) SELECT chart, count, search_terms, search_engine, url, referrer FROM `%1$s%2$s`', KSTATS_TABLE_CHARTS, $tempkey );


	foreach ( $query as $q )
		$wpdb->query( $q );

}


/**
 * Responsible for updating the kStats Top Referrers aggregate information.
 * Using a temporary table, the function retrieves its data in the same method as
 * used by the kStats Views, and replaces the current date in our totals table to
 * reflect the updated information.
 *
 * @category Cleanup
 */

function kstats_cleanup_referrers() {
	global $wpdb, $kstats;

	$tempkey = kstats_create_temp_key();

	$query[] = sprintf( 'CREATE TEMPORARY TABLE `%1$s%2$s`
					     SELECT %3$d AS chart, SUM( v.count ) AS count, v.referrer, v.url
						 FROM ( SELECT count( referrer ) AS count, referrer, url 
								FROM `%4$s`
								WHERE referrer NOT LIKE \'%5$s%%\' AND referrer <> \'\' AND search_engine = \'\' AND url <> \'\' AND preserved IS NULL
								GROUP BY referrer
								UNION ALL
								SELECT count, referrer, url
								FROM `%1$s`
								WHERE chart = %3$d )
						 AS v
						 GROUP BY v.referrer
						 ORDER BY count DESC', KSTATS_TABLE_CHARTS, $tempkey, KSTATS_CHARTS_REFERRERS, KSTATS_TABLE_RAW, $kstats->blog_url );

	$query[] = sprintf( "DELETE FROM `%s` WHERE chart = %d", KSTATS_TABLE_CHARTS, KSTATS_CHARTS_REFERRERS );

	$query[] = sprintf( 'INSERT INTO `%1$s` ( chart, count, referrer, url ) SELECT chart, count, referrer, url FROM `%1$s%2$s`', KSTATS_TABLE_CHARTS, $tempkey );


	foreach ( $query as $q )
		$wpdb->query( $q );

}


/**
 * Responsible for updating the kStats Top User Agents aggregate information.
 * Using a temporary table, the function retrieves its data in the same method as
 * used by the kStats Views, and replaces the current date in our totals table to
 * reflect the updated information.
 *
 * @category Cleanup
 */

function kstats_cleanup_agents() {
	global $wpdb;

	$tempkey = kstats_create_temp_key();

	$query[] = sprintf( 'CREATE TEMPORARY TABLE `%1$s%2$s`
					     SELECT %3$d AS chart, SUM( v.count ) AS count, v.user_agent, v.os, v.browser
						 FROM ( SELECT count( user_agent ) AS count, user_agent, os, browser
								FROM `%4$s`
								WHERE user_agent <> \'\' AND CONCAT( os, browser ) <> \'\' AND preserved IS NULL
								GROUP BY user_agent
								UNION ALL
								SELECT count, user_agent, os, browser
								FROM `%1$s`
								WHERE chart = %3$d )
						 AS v
						 GROUP BY v.user_agent
						 ORDER BY count DESC', KSTATS_TABLE_CHARTS, $tempkey, KSTATS_CHARTS_AGENTS, KSTATS_TABLE_RAW );

	$query[] = sprintf( "DELETE FROM `%s` WHERE chart = %d", KSTATS_TABLE_CHARTS, KSTATS_CHARTS_AGENTS );

	$query[] = sprintf( 'INSERT INTO `%1$s` ( chart, count, user_agent, os, browser ) SELECT chart, count, user_agent, os, browser FROM `%1$s%2$s`', KSTATS_TABLE_CHARTS, $tempkey );


	foreach ( $query as $q )
		$wpdb->query( $q );

}


/**
 * Responsible for updating the kStats Top Crawlers aggregate information.
 * Using a temporary table, the function retrieves its data in the same method as
 * used by the kStats Views, and replaces the current date in our totals table to
 * reflect the updated information.
 *
 * @category Cleanup
 */

function kstats_cleanup_spiders() {
	global $wpdb;

	$tempkey = kstats_create_temp_key();

	$query[] = sprintf( 'CREATE TEMPORARY TABLE `%1$s%2$s`
					     SELECT %3$d AS chart, SUM( v.count ) AS count, v.user_agent, v.spider, v.url
						 FROM ( SELECT COUNT( user_agent ) AS count, user_agent, spider, url
								FROM `%4$s`
								WHERE spider <> \'\' AND url <> \'\' AND url <> \'/robots.txt\' AND preserved IS NULL
								GROUP BY user_agent
								UNION ALL
								SELECT count, user_agent, spider, url
								FROM `%1$s`
								WHERE chart = %3$d )
						 AS v
						 GROUP BY v.user_agent
						 ORDER BY count DESC', KSTATS_TABLE_CHARTS, $tempkey, KSTATS_CHARTS_SPIDERS, KSTATS_TABLE_RAW );

	$query[] = sprintf( "DELETE FROM `%s` WHERE chart = %d", KSTATS_TABLE_CHARTS, KSTATS_CHARTS_SPIDERS );

	$query[] = sprintf( 'INSERT INTO `%1$s` ( chart, count, user_agent, spider, url ) SELECT chart, count, user_agent, spider, url FROM `%1$s%2$s`', KSTATS_TABLE_CHARTS, $tempkey );


	foreach ( $query as $q )
		$wpdb->query( $q );

}


/**
 * Deprecated. Keeping around for now in case I see another use for it somewhere.
 * Possibly inside the cleanup function? Need to use REPLACE though and not INSERT
 *
 * @ver 0.1.0
 * @category Cleanup
 * @param string $table  The name of the table we're running the query on
 * @param array  $fields An indexed array of fields to use in the query
 * @param array  $values An indexed array of preformatted values to insert into their respective fields
 * @param int    $per    How many sets of data to run in one query
 * @return bool True on success, false on failure
 */

function kstats_cleanup_run_query( $table, $fields = array(), $values = array(), $per = 200 ) {
	global $wpdb;

	if ( empty( $table ) || empty( $fields ) || empty( $values ) )
		return FALSE;

	$total_rows = count( $values );

	// Are there more rows to add than are allowed in one query?
	if ( $total_rows > $per ) {
		$iterations = ceil( $total_rows / $per );

		// Chop them up into little bits
		for ( $i = 1; $i <= $iterations; $i++ ) {
			$start = ( $i - 1 ) * $per;
			$query_parts[] = array_slice( $values, $start, $per );
		}

	}
	else
		$query_parts[] = $values;

	foreach ( $query_parts as $query ) {
		$query = sprintf( "INSERT INTO `%s` (%s) VALUES %s", $table, implode( ', ', $fields ), implode( ', ', $query ) );
		$wpdb->query( $query );
	}

	return TRUE;

}
